@@ -1,7 +1,10 @@
 aov2 = function(Formula, Data, eps=1e-8)
 {
-  y = model.frame(Formula, Data)[,1]
+  if (!attr(terms(Formula, data=Data), "response")) stop("Dependent variable should be provided!")  
   x = ModelMatrix(Formula, Data, KeepOrder=FALSE)
+  y = model.frame(Formula, Data)[,1]
+  if (!is.numeric(y)) stop("Dependent variable should be numeric!")
+
   r1 = lfit(x, y, eps=eps)
   T1 = SS(x, r1, e2(Formula, Data, eps=eps))
   if ("(Intercept)" %in% colnames(x$X)) {
