@@ -1,7 +1,7 @@
 estmb = function(L, X, g2, eps=1e-8) # Estimability check
 { # Reference: Kennedy & Gentle. Statistical Computing (1980) p361 
   nc = ncol(L)
-  if (nc != ncol(X) | nc != ncol(g2)) stop ("Matrix dimension mismatch!")
+  if (nc != ncol(X) | nc != NCOL(g2)) stop ("Matrix dimension mismatch!")
   L2 = L %*% g2 %*% crossprod(X)
   nr = nrow(L)
   vL = vector(length=nr)
